
//  VHS Cylon Paper towel Dispenser V6.0
//  Code to play all sound files and change LED colors (Red/Green/Blue/Pink/Orange)
//  Grabbed from all over the place (not really proud of my bit but it works)
//  https://github.com/LetsCodeBlacksburg/arduino-talking-skull
//  https://learn.adafruit.com/larson-scanner-shades/arduino-code
//  Sound track is triggered by grounding A4 via Opto isolator that is powered from motor of dispenser
//  Used a FOD817B opto to ground A4 driven from motor power on dispenser via 470R resistor
//  Pushbutton to ground also wired to A4 for manual trigger
//  Wiring based on various sites also
//  https://wiki.dfrobot.com/DFPlayer_Mini_SKU_DFR0299
//  https://www.electronics-lab.com/project/mp3-player-using-arduino-dfplayer-mini/
//  Create single folder on SD card named "01"
//  Store all MP3 audio files in it named consecitivley from "001.mp3"
//  Set fileCount value to number of audio files you have stored on SD card
//  Set setVol value for desired volume (from 0 to 30)
//  Make sure you install the Adafruit NeoPixel and the DFRobotDFPlayerMini libraries via the Arduino IDE Manage libraries function


#include "Arduino.h"
#include <SoftwareSerial.h>
#include <DFRobotDFPlayerMini.h>
#include <Adafruit_NeoPixel.h>

// Trigger Settings
const int TRIGGER = A4; // Pin for trigger
int val = 0;     // variable for reading the pin status


// Pixel Settings
const int PIN = 6;  // Pin for WS2811 pixels
int N_LEDS = 33;    // Number of pixels
int SPEED = 30;    // LED scanning speed (higher = slower)
int K = 1;          // Pixel color pointer

// DFPlayer Settings
const int ardRX = 10;   // The arduino software Receive lins (goes to DFPlayer TX)
const int ardTX = 11;   // The arduino software Transmit line (goes to DFPlayer RX)
const int dfBusy = A3;  // Coming from DFPlayer pin 16.. is active low (HIGH = not playing a sound)
int sndFile = 1;        // Sound file pointer.
int fileCount = 38;     // Set the max # of files here
int setVol = 28;        //Set volume value (0~30)

// Software Serial Pins To DFPlayer
SoftwareSerial mySoftwareSerial(10, 11);  // RX, TX

// Setup DFPlayer instance
DFRobotDFPlayerMini myDFPlayer;
void printDetail(uint8_t type, int value);  // Print status

// Setup NeoPixel instance
Adafruit_NeoPixel strip = Adafruit_NeoPixel(N_LEDS, PIN, NEO_GRB + NEO_KHZ800);

// *********************** SETUP *************************

void setup() {

  //  Set up DFPlay mini
  delay(250);
  pinMode(ardRX, INPUT);
  pinMode(ardTX, OUTPUT);
  pinMode(dfBusy, INPUT);
  delay(100);
  mySoftwareSerial.begin(9600);
  delay(100);

  Serial.begin(9600);
  Serial.println(F("VHS Cylon Paper Towel Dispenser V6.0"));  // Print some info
  Serial.println(F("DFPlayer Initializing"));

  // Check for DFPlay initialization via softserial
  if (!myDFPlayer.begin(mySoftwareSerial)) {    //Use softwareSerial to communicate with mp3.
    // Run this if it doesn't init
    delay(20);
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
    // Print some troubleshooting info that may help
    Serial.print("INFO: player state / file counts: ");
    Serial.print(myDFPlayer.readState()); //read mp3 state
    Serial.print(" / ");
    Serial.println(myDFPlayer.readFileCounts()); //read all file counts in SD card
    while (true);     // Hang forever if error
  }

  // If serial setup worked
  Serial.println(F("DFPlayer OK"));
  // configure settings
  myDFPlayer.setTimeOut(500);               //Set serial communictaion time out 500ms
  myDFPlayer.volume(setVol);                //Set volume value (0~30).
  myDFPlayer.EQ(DFPLAYER_EQ_BASS);          //Set EQ to bassy
  myDFPlayer.outputDevice(DFPLAYER_DEVICE_SD); // Set to use SD card

  // Print Status:
  Serial.print("DFPlayer Status: ");
  Serial.print(myDFPlayer.readState());         //read mp3 state
  Serial.print(" / Volume: ");
  Serial.print(myDFPlayer.readVolume());        //read current volume
  Serial.print(" / EQ: ");
  Serial.print(myDFPlayer.readEQ());          //read EQ setting
  Serial.print(" / SD Card File Count: ");
  Serial.println(myDFPlayer.readFileCounts());  //read all file counts in SD card

  pinMode(TRIGGER, INPUT_PULLUP); //Setup trigger input
  pinMode(LED_BUILTIN, OUTPUT); // Setup builtin LED

  strip.begin(); // Setup NeoPixels
  strip.show();  // Turn off all LEDs



}

int pos = 0, dir = 1; // Position, direction of "eye"  // setup initial eye position




// *********************** MAIN LOOP *************************

void loop()
{

delay(250);  // Delay for start

  // If at the last sound file, then loop back to the top (1)
  if (sndFile == (fileCount + 1) ) {
    sndFile = 1;
  }


  val = digitalRead(TRIGGER);  // read input value
  if (val == LOW) {         // check if the input is LOW (button pushed)
    digitalWrite(LED_BUILTIN, HIGH);  //turn on LED if triggered
    Serial.print("DFPlayer Triggered - Playing Track #: ");
    Serial.println(sndFile);
    myDFPlayer.play(sndFile);   //Play the next mp3
    delay(100);                 // wait to start

 // While DFPlayer _BUSY line is active(low), run NeoPixel Larson Scanner code, change color each run

    while (!digitalRead(dfBusy) == true ) {
      if (K == 1) {  // Use Red color
        int j;
        strip.setPixelColor(pos - 2, 0x100000); // Dark
        strip.setPixelColor(pos - 1, 0x800000); // Medium
        strip.setPixelColor(pos    , 0xFF2000); // Center pixel is brightest
        strip.setPixelColor(pos + 1, 0x800000); // Medium
        strip.setPixelColor(pos + 2, 0x100000); // Dark
        strip.show(); // Turn off all LEDs
        delay(SPEED); // Adjust for LED scanning speed
        for (j = -2; j <= 2; j++) strip.setPixelColor(pos + j, 0);
        // Bounce off ends of strip
        pos += dir;
        if (pos < 0) {
          pos = 1;
          dir = -dir;
        } else if (pos >= strip.numPixels()) {
          pos = strip.numPixels() - 2;
          dir = -dir;
        }
      }


      else if (K == 2) {  // Use Green color
        int j;
        strip.setPixelColor(pos - 2, 0x001000); // Dark
        strip.setPixelColor(pos - 1, 0x008000); // Medium
        strip.setPixelColor(pos    , 0x00FF10); // Center pixel is brightest
        strip.setPixelColor(pos + 1, 0x008000); // Medium
        strip.setPixelColor(pos + 2, 0x001000); // Dark
        strip.show(); // Turn off all LEDs
        delay(SPEED); // Adjust for LED scanning speed
        for (j = -2; j <= 2; j++) strip.setPixelColor(pos + j, 0);
        // Bounce off ends of strip
        pos += dir;
        if (pos < 0) {
          pos = 1;
          dir = -dir;
        } else if (pos >= strip.numPixels()) {
          pos = strip.numPixels() - 2;
          dir = -dir;
        }
      }


      else if (K == 3) {// Use Blue color
        int j;
        strip.setPixelColor(pos - 2, 0x000010); // Dark
        strip.setPixelColor(pos - 1, 0x000080); // Medium
        strip.setPixelColor(pos    , 0x2000FF); // Center pixel is brightest
        strip.setPixelColor(pos + 1, 0x000080); // Medium
        strip.setPixelColor(pos + 2, 0x000010); // Dark
        strip.show(); // Turn off all LEDs
        delay(SPEED); // Adjust for LED scanning speed
        for (j = -2; j <= 2; j++) strip.setPixelColor(pos + j, 0);
        // Bounce off ends of strip
        pos += dir;
        if (pos < 0) {
          pos = 1;
          dir = -dir;
        } else if (pos >= strip.numPixels()) {
          pos = strip.numPixels() - 2;
          dir = -dir;
        }
      }


      else if (K == 4) {// Use Pink color
        int j;
        strip.setPixelColor(pos - 2, 0x090006); // Dark
        strip.setPixelColor(pos - 1, 0x400020); // Medium
        strip.setPixelColor(pos    , 0xE00020); // Center pixel is brightest
        strip.setPixelColor(pos + 1, 0x400020); // Medium
        strip.setPixelColor(pos + 2, 0x090006); // Dark
        strip.show(); // Turn off all LEDs
        delay(SPEED); // Adjust for LED scanning speed
        for (j = -2; j <= 2; j++) strip.setPixelColor(pos + j, 0);
        // Bounce off ends of strip
        pos += dir;
        if (pos < 0) {
          pos = 1;
          dir = -dir;
        } else if (pos >= strip.numPixels()) {
          pos = strip.numPixels() - 2;
          dir = -dir;
        }
      }



      else if (K == 5) {// Use Orange color
        int j;
        strip.setPixelColor(pos - 2, 0x100400); // Dark
        strip.setPixelColor(pos - 1, 0x802000); // Medium
        strip.setPixelColor(pos    , 0xF02000); // Center pixel is brightest
        strip.setPixelColor(pos + 1, 0x802000); // Medium
        strip.setPixelColor(pos + 2, 0x100400); // Dark
        strip.show(); // Turn off all LEDs
        delay(SPEED); // Adjust for LED scanning speed
        for (j = -2; j <= 2; j++) strip.setPixelColor(pos + j, 0);
        // Bounce off ends of strip
        pos += dir;
        if (pos < 0) {
          pos = 1;
          dir = -dir;
        } else if (pos >= strip.numPixels()) {
          pos = strip.numPixels() - 2;
          dir = -dir;
        }
      }

      
      else if (K == 6) {// Use White color
        int j;
        strip.setPixelColor(pos - 2, 0x080808); // Dark
        strip.setPixelColor(pos - 1, 0x505050); // Medium
        strip.setPixelColor(pos    , 0xDFDFDF); // Center pixel is brightest
        strip.setPixelColor(pos + 1, 0x505050); // Mediu
        strip.setPixelColor(pos + 2, 0x080808); // Dar
        strip.show(); // Turn off all LEDs
        delay(SPEED); // Adjust for LED scanning speed
        for (j = -2; j <= 2; j++) strip.setPixelColor(pos + j, 0);
        // Bounce off ends of strip
        pos += dir;
        if (pos < 0) {
          pos = 1;
          dir = -dir;
        } else if (pos >= strip.numPixels()) {
          pos = strip.numPixels() - 2;
          dir = -dir;
        }
      }


      
    }


    K++;  // Increment color pointer
    if (K > 6) {
      K = 1;
    }


    Serial.print("Current LED Position: ");
    Serial.println(pos);

    
    strip.show(); // Turn off all LEDs
    sndFile++;     // Incriment track index to the next file

    // if button not pused then turn the LED off
  } else {
    digitalWrite(LED_BUILTIN, LOW);
  }

}



//printDetail() - Shows DF Player status

void printDetail(uint8_t type, int value) {

  switch (type) {
    case TimeOut:
      Serial.println(F("Time Out!"));
      break;
    case WrongStack:
      Serial.println(F("Stack Wrong!"));
      break;
    case DFPlayerCardInserted:
      Serial.println(F("Card Inserted!"));
      break;
    case DFPlayerCardRemoved:
      Serial.println(F("Card Removed!"));
      break;
    case DFPlayerCardOnline:
      Serial.println(F("Card Online!"));
      break;
    case DFPlayerPlayFinished:
      Serial.print(F("Number:"));
      Serial.print(value);
      Serial.println(F(" Play Finished!"));
      break;
    case DFPlayerError:
      Serial.print(F("DFPlayerError:"));
      switch (value) {
        case Busy:
          Serial.println(F("Card not found"));
          break;
        case Sleeping:
          Serial.println(F("Sleeping"));
          break;
        case SerialWrongStack:
          Serial.println(F("Get Wrong Stack"));
          break;
        case CheckSumNotMatch:
          Serial.println(F("Check Sum Not Match"));
          break;
        case FileIndexOut:
          Serial.println(F("File Index Out of Bound"));
          break;
        case FileMismatch:
          Serial.println(F("Cannot Find File"));
          break;
        case Advertise:
          Serial.println(F("In Advertise"));
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }
}
